@@ -7,33 +7,58 @@ from .config import *
 
 
 def common_taxids():
-    """ Return taxonomy IDs for most common organisms. """
+    """ Return taxonomy IDs for most common organisms.
+
+    :rtype: :class:`list` of common organisms
+    """
     return [tax_id for tax_id, _ in COMMON_NAMES]
 
 
 def common_taxid_to_name(tax_id):
-    """ Return a name for a common organism taxonomy id. """
+    """ Return a name for a common organism taxonomy id.
+
+    :param tax_id: Taxonomy if (NCBI taxonomy database)
+    :type tax_id: str
+
+    :rtype: :class:`str`
+    """
+
     return COMMON_NAMES_MAPPING[str(tax_id)]
 
 
-def taxname_to_taxid(name):
-    """ Return taxonomy ID for a taxonomy name. """
+def taxname_to_taxid(organism_name):
+    """ Return taxonomy ID for a taxonomy name.
+
+    :param organism_name: Official organism name, e.g., 'Homo sapiens'
+    :type organism_name: str
+
+    :rtype: :class:`str`
+    """
     name_to_taxid = dict(map(reversed, COMMON_NAMES))
 
-    if name in name_to_taxid:
-        return name_to_taxid[name]
+    if organism_name in name_to_taxid:
+        return name_to_taxid[organism_name]
     return None
 
 
-def shortname(taxid):
-    """ Short names for common_taxids organisms """
-    if taxid in SHORT_NAMES:
-        return SHORT_NAMES[taxid]
+def shortname(tax_id):
+    """ Short names for common_taxids organisms
+
+    :param tax_id: Taxonomy if (NCBI taxonomy database)
+    :type tax_id: str
+
+    """
+    if tax_id in SHORT_NAMES:
+        return SHORT_NAMES[tax_id]
     return []
 
 
 def name(tax_id):
     """ Return the scientific name for organism with provided taxonomy id.
+
+    :param tax_id: Taxonomy if (NCBI taxonomy database)
+    :type tax_id: str
+
     """
     # Most of the lookups will be for the common names, so in most
     # situations we can avoid loading the taxonomy.
@@ -45,7 +70,12 @@ def name(tax_id):
 
 def other_names(tax_id):
     """ Return a list of (name, name_type) tuples excluding the scientific name.
+
     Use :func:`name` to retrieve the scientific name.
+
+    :param tax_id: Taxonomy if (NCBI taxonomy database)
+    :type tax_id: str
+
     """
     return Taxonomy().other_names(tax_id)
 
@@ -54,17 +84,26 @@ def search(string, onlySpecies=True, exact=False):
     """ Search the NCBI taxonomy database for an organism.
 
     :param string: Search string.
+    :type string: str
+
     :param onlySpecies: Return only taxids of species (and subspecies).
+    :type onlySpecies: bool
+
     :param exact:  Return only taxids of organism that exactly match the string.
+    :type exact: bool
     """
     ids = Taxonomy().search(string, onlySpecies, exact)
     return list(ids)
 
 
-def lineage(taxid):
+def lineage(tax_id):
     """ Return a list of taxids ordered from the topmost node (root) to taxid.
+
+    :param tax_id: Taxonomy if (NCBI taxonomy database)
+    :type tax_id: str
     """
-    return Taxonomy().lineage(taxid)
+
+    return Taxonomy().lineage(tax_id)
 
 
 def to_taxid(code, mapTo=None):
@@ -112,4 +151,5 @@ def to_taxid(code, mapTo=None):
 def ensure_downloaded(callback=None, verbose=True):
     """ Retrieve the taxonomy database if not already downloaded.
     """
+    warnings.warn("'ensure_downloaded' is deprecated", DeprecationWarning)
     serverfiles.localpath_download(DOMAIN, FILENAME, callback=callback, verbose=verbose)
