@@ -1,43 +1,106 @@
 import math
+import scipy
 import threading
 import numpy as np
 
+from typing import Tuple, Union
 
 from scipy.stats import hypergeom
 
+ALT_TWO = "two-sided"
+ALT_LESS = "less"
+ALT_GREATER = "greater"
+ALTERNATIVES = [ALT_GREATER, ALT_TWO, ALT_LESS]
 
-def hypergeometric_test(X, cluster, treshold):
+def score_t_test(a, b, **kwargs):
+    # type: (np.array, np.array) -> Tuple[Union[float, np.array], Union[float, np.array]]
+    """ Run t-test. Enable setting different alternative hypothesis.
+    Probabilities are exact due to symmetry of the test.
+
+    :returns (statistics, p_values)
+
+    See also
+    --------
+    scipy.stats.ttest_ind
+
+    """
+    alt = kwargs.get("alternative", ALT_TWO)
+    assert alt in ALTERNATIVES
+    scores, pvalues = scipy.stats.ttest_ind(a, b, axis=kwargs.get('axis', 0))
+    if alt == ALT_TWO:
+        return scores, pvalues
+    less = scores < 0
+    pvalues = pvalues / 2.0
+    pvalues[np.logical_not(less)] = 1.0 - pvalues[np.logical_not(less)]
+    if alt == ALT_LESS:
+        return scores, pvalues
+    else:
+        return scores, 1.0 - pvalues
+
+def score_mann_whitney(a, b, **kwargs):
+    axis = kwargs.get('axis', 0)
+    a, b = np.asarray(a, dtype=float), np.asarray(b, dtype=float)
+
+    if not 0 <= axis < 2:
+        raise ValueError("Axis")
+
+    if a.ndim != b.ndim:
+        raise ValueError
+
+    if axis >= a.ndim:
+        raise ValueError
+
+    if axis == 0:
+        a, b = a.T, b.T
+
+    alt = kwargs.get("alternative", ALT_TWO)
+    assert alt in ALTERNATIVES
+    statistics = np.zeros((a.shape[0]),)
+    p_values = np.zeros((a.shape[0]),)
+    for i, (a_, b_) in enumerate(zip(a, b)):
+        try:
+            s, p = scipy.stats.mannwhitneyu(a_, b_, alternative=alt)
+        except ValueError:
+            s, p = (0, 1)
+        statistics[i] = s
+        p_values[i] = p
+    return np.array(statistics), np.array(p_values)
+
+
+def score_hypergeometric_test(a, b, threshold=1, **kwargs):
+    """
+    Run a hypergeometric test. The probability in a two-sided test is approximated
+    with the symmetric distribution with more extreme of the tails.
+    """
     # type: (np.ndarray, np.ndarray, float) -> np.ndarray
 
-    scores = np.zeros((X.shape[1],))
+    # Binary expression matrices
+    A = (a >= threshold).astype(int)
+    B = (b >= threshold).astype(int)
+    alt = kwargs.get("alternative", ALT_TWO)
+    assert alt in ALTERNATIVES
 
-    # Binary expression matrix
-    Y = (X >= treshold).astype(int)
+    # Test Parameters
+    M = len(A) + len(B)
+    N = len(A)
+    n_expr = A.sum(axis=0) + B.sum(axis=0)  # Number of cells expressing genes (overall)
+    n_expr_clust = A.sum(axis=0)            # Number of cells expressing genes (in cluster)
 
-    # Process each gene
-    for gi, g in enumerate(Y.T):
-        # Test parameters
-        M = X.shape[0]  # Number of cells
-        n = g.sum()  # Number of cells expressing g
-        N = len(cluster)  # Number of cells belonging to cluster(s)
-        hg = hypergeom(M, n, N)
-
-        # Test for over expression
-        x = g[cluster].sum()
-        x_over = np.arange(x, n + 1)  # x or more
-        pvalue_over = hg.pmf(x_over).sum()
-
-        # Test for under expression
-        x_under = np.arange(0, x + 1)  # x or less
-        pvalue_under = hg.pmf(x_under).sum()
-
-        # Proposed scoring:
-        p = min(pvalue_under, pvalue_over)
-        s = -1 if pvalue_under < pvalue_over else 1
-        score = -np.log(p) * s
-        scores[gi] = score
-
-    return scores
+    # Test results --- both tails
+    # Note: cumulatives do sum to >1 due to overlap at 1 point
+    under = np.fromiter(map(lambda t: hypergeom.cdf(k=t[1], n=t[0], M=M, N=N),
+                            zip(n_expr, n_expr_clust)), dtype=float)
+    over = np.fromiter(map(lambda t: hypergeom.sf(k=t[1]-1, n=t[0], M=M, N=N),
+                           zip(n_expr, n_expr_clust)), dtype=float)
+    signs = np.sign(under - over)
+    if alt == ALT_TWO:
+        pvalues = np.minimum(1.0, 2.0 * np.minimum(under, over))
+    elif alt == ALT_LESS:
+        pvalues = under
+    else:
+        pvalues = over
+    scores = -np.log(pvalues) * signs
+    return scores, pvalues
 
 
 def _lngamma(z):
@@ -126,7 +189,7 @@ class Binomial(LogBin):
     def p_value(self, k, N, m, n):
         """ The probability that k or more tests are positive. """
         if n - k + 1 <= k:
-            #starting from k gives the shorter list of values
+            # starting from k gives the shorter list of values
             return sum(self.__call__(i, N, m, n) for i in range(k, n+1))
         else:
             value = 1.0 - sum(self.__call__(i, N, m, n) for i in range(k))
