@@ -25,7 +25,7 @@ GENE_ID_ATTRIBUTE = 'gene_id_attribute'
 
 # Error strings
 ERROR_ON_MISSING_ANNOTATION = 'Missing annotation on gene IDs and organism in the input data.'
-ERROR_ON_MISSING_GENE_ID = 'Missing Gene ID information in the input data'
+ERROR_ON_MISSING_GENE_ID = 'Missing gene ID information in the input data'
 ERROR_ON_MISSING_TAX_ID = 'Missing organism information in the input data'
 
 col_spec = Sequence[Tuple[Orange.data.Variable, Sequence[numbers.Real]]]
